/* seek-dial.m (2024-01-06)

Rotates a seek needle.

Written by Ariszló <ariszlo@gmail.com> following FrisbeeMonkey’s example of
a volume knob using rotation functions from Winamp 3’s rotationlayer.m

Bug fixed by Defix <Defixmail@gmail.com>

This script requires a seek map image with the id "round.seek.map".
The seek map image is a greyscale image: darkest where the song begins and
lightest where it ends. It is of the same size as that of the seek.dial image.

The id of the image this script will rotate is "seek.dial" in the below example
but you can use whatever id you like. The image with the id "ring.around" is
optional: it hides the distortions caused by the rotation.

<elements>
  <bitmap id="seek.dial" file="?" x="?" y="?" w="?" h="?"/>
  <bitmap id="round.seek.map" file="?" x="?" y="?" w="?" h="?"/>
  <bitmap id="ring.around" file="?" x="?" y="?" w="?" h="?"/>
</elements>

Call this script somewhere after the following xml tags (replace the question
marks in the below examples with actual values):

  <Text
    id="InfoTicker"
    x="?" y="?" w="?" h="?"
    visible="0"
  />
  <SongTicker
    id="SongTicker"
    x="?" y="?" w="?" h="?"
  />
  <AnimatedLayer
    id="Seek Dial" image="seek.dial"
    x="?" y="?" w="?" h="?"
    move="0" autoplay="0" autoreplay="0"
  />
  <Layer
    id="Seek Dial Ring" image="ring.around"
    x="?" y="?" w="?" h="?" ghost="1"
  /> <!-- an optional layer to hide rotation-caused distortions -->

Call the script like this:

  <script id="seek-dial" file="scripts/seek-dial.maki"/> 
*/

// Never forget to include std.mi:
#include "lib/std.mi"

// Define PI up to 10 decimal places:
#ifndef PI
#define PI 3.1415926536
#endif

// Script variables used throughout this script:
Global AnimatedLayer SeekAnim;
Global Map SeekMap;
Global GuiObject SongTicker;
Global Text InfoTicker;
Global Timer TickerTimer;
Global Boolean SeekChanging;

// Rotation variable from Winamp 3’s rotationlayer.m:
Global Double dialR;

// Functions:
Function setSeekAnim(int posValue);
Function updateSeek(int x, int y);

// Functions from Winamp 3’s rotationlayer.m:
Function dialInit();
Function dialRotateDegree(double r);
Function dialRotate(double r);

// Do this when the script is loaded:
System.onScriptLoaded() {
  // Get the xml group calling this script:
  Group ScriptGrp = getScriptGroup();

  // Find the xml objects within the group:
  SeekAnim = ScriptGrp.findObject("Seek Dial");
  SongTicker = ScriptGrp.findObject("SongTicker");
  InfoTicker = ScriptGrp.findObject("Infoticker");

  SongTicker.show();
  InfoTicker.hide();

  // Initialize the script’s timer:
  TickerTimer = new Timer;
  TickerTimer.setDelay(1000);

  // Load the gradient map:
  SeekMap = new Map;
  SeekMap.loadMap("round.seek.map");

  // Initialize rotation:
  dialInit();

  // Position the needle:
  if (getStatus() == 0) setSeekAnim(0);
  else {
    // Add a conditional to avoid division by zero:
    if (getPlayItemLength() > 0) {
      // Start the timer:
      TickerTimer.start();
      // Send progress info to the setSeekAnim function:
      setSeekAnim(getPosition()/getPlayItemLength());
    }
    else setSeekAnim(0);
  }
}

// Start the timer when playback starts or resumes:
System.onPlay() {
  TickerTimer.start();
}

System.onResume() {
  TickerTimer.start();
}

// Stop the timer when playback is stopped:
System.onStop() {
  TickerTimer.stop();
  setSeekAnim(0);
}

// Send progress info to the setSeekAnim function:
TickerTimer.onTimer() {
  if (System.getPlayItemLength() > 0) {
    setSeekAnim(getPosition()/getPlayItemLength());
  }
  else {
    setSeekAnim(0);
  }
}

// Tell updateSeek that the mouse is pressed and update the animation:
SeekAnim.onLeftButtonDown(int x, int y) {
  TickerTimer.stop();
  SeekChanging = 1;
  updateSeek(x, y);
}

// Tell updateSeek that the mouse is released and update the animation:
SeekAnim.onLeftButtonUp(int x, int y) {
  TickerTimer.start();
  if (SeekChanging) {
    SeekChanging = 0;
    updateSeek(x, y);
  }
}

// Do this when dragging the needle around:
SeekAnim.onMouseMove(int x, int y) {
  if (SeekChanging) {
    updateSeek(x, y);
  }
}

// This is the function that calculates the needle position:
updateSeek(int x, int y) {
  int newValue = SeekMap.getValue(x - SeekAnim.getLeft(), y - SeekAnim.getTop());
  if (System.getPlayItemLength() > 0) {
    int p = (newValue * 100) / 255;
    int s = (newValue * System.getPlayItemLength()) / 255;
    // For songs shorter than 1 hour, display the current position and the
    // song's length in MM:SS format: 
    if (getPlayItemLength() < 3600000) {
      InfoTicker.setText(integerToTime(s) + " / " + integerToTime(getPlayItemLength()) + "   " + integerToString(p) + "%");
    }
    // For songs longer than 1 hour, display the current position and the
    // song's length in HH:MM:SS format: 
    else {
      InfoTicker.setText(integerToLongTime(s) + " / " + integerToLongTime(getPlayItemLength()) + "   " + integerToString(p) + "%");
    }
    // Show seek info while dragging the needle:
    if (SeekChanging) {
      SongTicker.hide();
      InfoTicker.show();
    }
    // Continue playing the song from where the button was released:
    if (!SeekChanging) {
      System.seekTo(s);
      InfoTicker.hide();
      SongTicker.show();
    }
    setSeekAnim(s/getPlayItemLength());
  }
}

// And this does the rotation:
setSeekAnim(float Value) {
  Value = Value*360;
  dialRotateDegree(-Value);
}

// Initialize layer fx settings:
dialInit() {
  SeekAnim.fx_setGridSize(10,10);
  SeekAnim.fx_setBgFx(0);
  SeekAnim.fx_setWrap(0);
  SeekAnim.fx_setBilinear(1);
  SeekAnim.fx_setRect(0);
  SeekAnim.fx_setClear(0);
  SeekAnim.fx_setLocalized(1);
  SeekAnim.fx_setRealtime(0);
  SeekAnim.fx_setEnabled(1);
}

// Convert degrees to radians, then call dialRotate():
dialRotateDegree(double r) {
  dialRotate(r * PI / 180.0);
}

// Rotate the SeekAnim layer:
dialRotate(double r) {
  dialR = r;
  SeekAnim.fx_update();
}

// Do the actual rotation of the layer:
SeekAnim.fx_onGetPixelR(double r, double d, double x, double y) {
  return r + dialR;
}

// Do this before the script is unloaded:
System.onScriptUnloading() {
  delete TickerTimer;
  delete SeekMap;
}
