/* seek-regions.m (2024-01-15)

FrisbeeMonkey’s seekbyregion.m adapted for modern SongTicker GUI objects
by Ariszló <ariszlo@gmail.com>

2024-01-06: turned seek animation off for streams.

This script requires a seek map image with two different object id’s:
"seek.map" reacting to mouse events, and "seek.map2" calculating how much of
the seek animation image should be made visible. A seek image is a greyscale
image: darkest where the song starts and lightest where it ends. It is of the
same size as that of the seek animation image.

The id of the image whose regions this script will reveal is "seek.regions"
in the below example but you can use whatever id you like.

<elements>
  <bitmap id="seek.regions" file="?" x="?" y="?" w="?" h="?"/>
  <bitmap id="seek.map" file="?" x="?" y="?" w="?" h="?"/>
  <bitmap id="seek.map2" file="?" x="?" y="?" w="?" h="?"/>
</elements>

Call this script somewhere after the following xml tags (replace the question
marks in the below examples with actual values):

  <Text
    id="InfoTicker"
    x="?" y="?" w="?" h="?"
    visible="0"
  />
  <SongTicker
    id="SongTicker"
    x="?" y="?" w="?" h="?"
  />
  <AnimatedLayer
    id="Seek Regions" image="seek.regions"
    x="?" y="?" w="?" h="?"
    move="0" autoplay="0" autoreplay="0"
  />

Call the script like this:

  <script id="seek-regions" file="scripts/seek-regions.maki"/> 

FrisbeeMonkey’s original seekbyregion.m is archived here:

https://web.archive.org/web/20160401195831/http://frisbeemonkey.com/winamp3/references/maki.php?offset=10&sorter=name
*/

// Never forget to include std.mi:
#include "lib/std.mi"

// Script variables used throughout this script:
Global AnimatedLayer SeekAnim;
Global Map SeekMap, SeekMap2;
Global GuiObject SongTicker;
Global Text InfoTicker;
Global Timer TickerTimer;
Global Boolean SeekChanging;

// Functions:
Function setSeekAnim(int posValue);
Function updateSeek(int x, int y);

// Do this when the script is loaded:
System.onScriptLoaded() {
  // Get the xml group calling this script:
  Group ScriptGrp = getScriptGroup();

  // Find the xml objects within the group:
  SeekAnim = ScriptGrp.findObject("Seek Regions");
  SongTicker = ScriptGrp.findObject("SongTicker");
  InfoTicker = ScriptGrp.findObject("Infoticker");

  SongTicker.show();
  InfoTicker.hide();

  // Initialize the script’s timer:
  TickerTimer = new Timer;
  TickerTimer.setDelay(1000);

  // Load the maps:
  SeekMap = new Map;
  SeekMap.loadMap("seek.map");
  SeekMap2 = new Map;
  SeekMap2.loadMap("seek.map2");

  // Do not show the seek animation when playback is stopped:
  if (getStatus() == 0) {
    setSeekAnim(0);
    SeekAnim.hide();
  }
  else {
    // Add a conditional to avoid division by zero:
    if (getPlayItemLength() > 0) {
      SeekAnim.show();
      // Start the timer:
      TickerTimer.start();
      // Send progress info to the setSeekAnim function:
      setSeekAnim(255 * System.getPosition() / System.getPlayItemLength());
    }
    else {
      setSeekAnim(0);
      SeekAnim.hide();
    }
  }
}

// Start the timer when playback starts or resumes:
System.onPlay() {
  TickerTimer.start();
}

System.onResume() {
  TickerTimer.start();
}

// Stop the timer when playback is stopped:
System.onStop() {
  TickerTimer.stop();
  setSeekAnim(0);
  SeekAnim.hide();
}

// Send progress info to the setSeekAnim function:
TickerTimer.onTimer() {
  if (System.getPlayItemLength() > 0) {
    SeekAnim.show();
    setSeekAnim(255 * System.getPosition() / System.getPlayItemLength());
  }
  else {
    setSeekAnim(0);
    SeekAnim.hide();
  }
}

// Show this much of the seek animation image:
setSeekAnim(int Value) {
  Region r = new Region;
  r.loadFromMap(SeekMap2, Value, 1);
  SeekAnim.setRegion(r);
  delete r;
}

// Tell updateSeek that the mouse is pressed and update the animation:
SeekAnim.onLeftButtonDown(int x, int y) {
  TickerTimer.stop();
  SeekChanging = 1;
  updateSeek(x, y);
}

// Tell updateSeek that the mouse is released and update the animation:
SeekAnim.onLeftButtonUp(int x, int y) {
  TickerTimer.start();
  if (SeekChanging) {
    SeekChanging = 0;
    updateSeek(x, y);
  }
}

// Do this while changing playback position with the mouse:
SeekAnim.onMouseMove(int x, int y) {
  if (SeekChanging) {
    updateSeek(x, y);
  }
}

// Update seek:
updateSeek(int x, int y) {
  int newValue = SeekMap.getValue(x - SeekAnim.getLeft(), y - SeekAnim.getTop());
  if (System.getPlayItemLength() > 0) {
    int p = (newValue * 100) / 255;
    int s = (newValue * System.getPlayItemLength()) / 255;
    // For songs shorter than 1 hour, display the current position and the
    // song's length in MM:SS format: 
    if (getPlayItemLength() < 3600000) {
      InfoTicker.setText(integerToTime(s) + " / " + integerToTime(getPlayItemLength()) + "   " + integerToString(p) + "%");
    }
    // For songs longer than 1 hour, display the current position and the
    // song's length in HH:MM:SS format: 
    else {
      InfoTicker.setText(integerToLongTime(s) + " / " + integerToLongTime(getPlayItemLength()) + "   " + integerToString(p) + "%");
    }
    // Show seek info while dragging the seeker:
    if (SeekChanging) {
      SongTicker.hide();
      InfoTicker.show();
    }
    // Continue playing the song from where the button was released:
    if (!SeekChanging) {
      System.seekTo(s);
      InfoTicker.hide();
      SongTicker.show();
    }
    // Calculate how much of the seek animation image should be shown:
    setSeekAnim(255 * s / System.getPlayItemLength());
  }
}

// Do this before the script is unloaded:
System.onScriptUnloading() {
  delete TickerTimer;
  delete SeekMap;
  delete SeekMap2;
}
