/* volume-knob.m (2024-01-02)

FrisbeeMonkey’s volumeknob.m adapted for modern SongTicker GUI objects
by Ariszló <ariszlo@gmail.com>

This script requires a volume map image with the id "volume.knob.map".
It is a greyscale image: darkest where the volume is lowest and lightest
where it is highest. It is of the same size as that of the volume.knob image.

The id of the image this script will rotate is "volume.knob" in the below
example but you can use whatever id you like. The image with the id
"ring.around" is optional: it hides the distortions caused by rotation.

<elements>
  <bitmap id="volume.knob" file="?" x="?" y="?" w="?" h="?"/>
  <bitmap id="volume.knob.map" file="?" x="?" y="?" w="?" h="?"/>
  <bitmap id="ring.around" file="?" x="?" y="?" w="?" h="?"/>
</elements>

Call this script somewhere after the following xml tags (replace the question
marks in the below examples with actual values):

  <Text
    id="InfoTicker"
    x="?" y="?" w="?" h="?"
    visible="0"
  />
  <SongTicker
    id="SongTicker"
    x="?" y="?" w="?" h="?"
  />
  <AnimatedLayer
    id="Volume Knob" image="volume.knob"
    x="?" y="?" w="?" h="?"
    move="0" autoplay="0" autoreplay="0"
  />
  <Layer
    id="Volume Knob Ring" image="ring.around"
    x="?" y="?" w="?" h="?" ghost="1"
  /> <!-- an optional layer to hide rotation-caused distortions -->

Call the script like this:

  <script id="volume-knob" file="scripts/volume-knob.maki"/> 

FrisbeeMonkey’s original volumeknob.m is archived here:

https://web.archive.org/web/20160311101829/http://frisbeemonkey.com/winamp3/references/maki.php?offset=20&sorter=name
*/

#include "lib/std.mi"

// Define PI up to 10 decimal places:
#ifndef PI
#define PI 3.1415926536
#endif

// Script variables used throughout this script:
Global AnimatedLayer VolumeAnim;
Global Map VolumeMap;
Global GuiObject SongTicker;
Global Text InfoTicker;
Global Boolean VolumeChanging;

// Rotation variable from Winamp 3’s rotationlayer.m:
Global Double KnobR;

// Functions:
Function updateVolume(int x, int y);
Function rotateKnob(float newvol);

// Functions from Winamp 3’s rotationlayer.m:
Function KnobInit();
Function KnobRotateDegree(double r);
Function KnobRotate(double r);

// Do this when the script is loaded:
System.onScriptLoaded() {
  // Get the xml group calling this script:
  Group ScriptGrp = getScriptGroup();

  // Find the xml objects within the group:
  VolumeAnim = ScriptGrp.findObject("Volume Knob");
  SongTicker = ScriptGrp.findObject("SongTicker");
  InfoTicker = ScriptGrp.findObject("Infoticker");

  SongTicker.show();
  InfoTicker.hide();

  // Load the gradient map:
  volumeMap = new Map;
  volumeMap.loadMap("volume.knob.map");

  // Initialize rotation:
  KnobInit();

  // Initialize volume variables and set knob position:
  VolumeChanging = 0;
  float u;
  u = System.getVolume();
  u = u / 255;
  rotateKnob(u);
}

// Tell the updateVolume(x, y) function that the mouse is pressed
// and update the volume:
VolumeAnim.onLeftButtonDown(int x, int y) {
  VolumeChanging = 1;
  updateVolume(x, y);
}

// Tell the updateVolume(x, y) function that the mouse is no longer pressed
// and update the volume:
VolumeAnim.onLeftButtonUp(int x, int y) {
  if (VolumeChanging) {
    VolumeChanging = 0;
    updateVolume(x, y);
  }
}

// Do this when dragging the knob around:
VolumeAnim.onMouseMove(int x, int y) {
  if (VolumeChanging) {
    updateVolume(x, y);
  }
}

// Calculate knob rotation and volume percentage when volume changes:
System.onVolumeChanged(int newVol) {
  Float f = getVolume();
  f = f / 255;
  rotateKnob(f);
  int p = (newVol * 100) / 255;
  InfoTicker.setText("Volume: " + System.integerToString(p) + "%");
}

// Update the volume:
updateVolume(int x, int y) {
  int newValue = VolumeMap.getValue(x - VolumeAnim.getLeft(), y - VolumeAnim.getTop());
  System.setVolume(newValue);
  if (VolumeChanging) {
    SongTicker.hide();
    InfoTicker.show();
  }
  if (!VolumeChanging) {
    InfoTicker.hide();
    SongTicker.show();
  }
}

// Rotate the knob to new orientation:
rotateKnob(float newvol){
  newvol = newvol * 330;
  KnobRotateDegree(-newvol);
}

// Initialize layer fx settings:
KnobInit() {
  VolumeAnim.fx_setGridSize(10,10);
  VolumeAnim.fx_setBgFx(0);
  VolumeAnim.fx_setWrap(0);
  VolumeAnim.fx_setBilinear(1);
  VolumeAnim.fx_setRect(0);
  VolumeAnim.fx_setClear(0);
  VolumeAnim.fx_setLocalized(1);
  VolumeAnim.fx_setRealtime(0);
  VolumeAnim.fx_setEnabled(1);
}

// Convert degrees to radians, then call KnobRotate():
KnobRotateDegree(double r) {
  KnobRotate(r * PI / 180.0);
}

// Rotate the VolumeAnim layer:
KnobRotate(double r) {
  KnobR = r;
  VolumeAnim.fx_update();
}

// Do the actual rotation of the layer:
VolumeAnim.fx_onGetPixelR(double r, double d, double x, double y) {
  return r + KnobR;
}

// Do this before the script is unloaded:
System.onScriptUnloading() {
  delete VolumeMap;
}
