/* volume-regions.m (2024-01-02)

FrisbeeMonkey’s volumebyregion.m adapted to modern SongTicker GUI objects
by Ariszló <ariszlo@gmail.com>

This script requires a volume map with two different id’s: "volume.map"
reacting to mouse events, and "volume.map2" calculating how much of the volume
animation image should be made visible. A volume image is a greyscale image:
darkest where the volume is lowest and lightest where it is highest. It is of
the same size as that of the volume animation image.

The id of the image whose regions this script will reveal is "volume.regions"
in the below example but you can use whatever id you like.

<elements>
  <bitmap id="volume.regions" file="?" x="?" y="?" w="?" h="?"/>
  <bitmap id="volume.map" file="?" x="?" y="?" w="?" h="?"/>
  <bitmap id="volume.map2" file="?" x="?" y="?" w="?" h="?"/>
</elements>

Call this script somewhere after the following xml tags (replace the question
marks in the below examples with actual values):

  <Text
    id="InfoTicker"
    x="?" y="?" w="?" h="?"
    visible="0"
  />
  <SongTicker
    id="SongTicker"
    x="?" y="?" w="?" h="?"
  />
  <AnimatedLayer
    id="Volume Regions" image="volume.regions"
    x="?" y="?" w="?" h="?"
    move="0" autoplay="0" autoreplay="0"
  />

Call the script like this:

  <script id="volume-regions" file="scripts/volume-regions.maki"/> 

FrisbeeMonkey’s original volumebyregion.m is archived here:

https://web.archive.org/web/20160311101829/http://frisbeemonkey.com/winamp3/references/maki.php?offset=20&sorter=name
*/

// Never forget to include std.mi:
#include "lib/std.mi"

// Script variables used throughout this script:
Global AnimatedLayer VolumeAnim;
Global Map VolumeMap, VolumeMap2;
Global GuiObject SongTicker;
Global Text InfoTicker;
Global Timer TickerTimer;
Global Boolean VolumeChanging;

// Functions:
Function setVolumeAnim(int volValue);
Function updateVolume(int x, int y);

// Do this when the script is loaded:
System.onScriptLoaded() {
  // Get the xml group calling this script:
  Group ScriptGrp = getScriptGroup();

  // Find the xml objects within the group:
  VolumeAnim = ScriptGrp.findObject("Volume Regions");
  SongTicker = ScriptGrp.findObject("SongTicker");
  InfoTicker = ScriptGrp.findObject("Infoticker");

  SongTicker.show();
  InfoTicker.hide();

  // Initialize the script’s timer:
  TickerTimer = new Timer;
  TickerTimer.setDelay(750);

  // Load the maps:
  VolumeMap = new Map;
  VolumeMap.loadMap("volume.map");
  VolumeMap2 = new Map;
  VolumeMap2.loadMap("volume.map2");

  // Send volume level to the setVolumeAnim() function:
  setVolumeAnim(System.getVolume());
}

// Do this when the volume changes:
System.onVolumeChanged(int newVol) {
  // Send volume level to the setVolumeAnim function:
  setVolumeAnim(newVol);
  // Calculate percentage:
  int p = (newVol * 100) / 255;
  // Show volume level in the info ticker:
  TickerTimer.start();
  SongTicker.hide();
  InfoTicker.show();
  InfoTicker.setText("Volume: " + System.integerToString(p) + "%");
}

// Show this much of the volume animation image:
setVolumeAnim(int Value) {
  Region r = new Region;
  r.loadFromMap(VolumeMap2, Value, 1);
  VolumeAnim.setRegion(r);
  delete r;
}

// Tell updateVolume that the mouse is pressed and update the animation:
VolumeAnim.onLeftButtonDown(int x, int y) {
  VolumeChanging = 1;
  updateVolume(x, y);
}

// Tell updateVolume that the mouse is released and update the animation:
VolumeAnim.onLeftButtonUp(int x, int y) {
  if (VolumeChanging) {
    VolumeChanging = 0;
    updateVolume(x, y);
  }
}

// Do this when changing the volume with the mouse:
VolumeAnim.onMouseMove(int x, int y) {
  if (VolumeChanging) {
    updateVolume(x, y);
  }
}

// Update the animation:
updateVolume(int x, int y) {
  int newValue = VolumeMap.getValue(x - VolumeAnim.getLeft(), y - VolumeAnim.getTop());
  System.setVolume(newValue);
}

// Show the song ticker again:
TickerTimer.onTimer() {
  SongTicker.show();
  InfoTicker.hide();
  TickerTimer.stop();
}

// Do this before the script is unloaded:
System.onScriptUnloading() {
  delete TickerTimer;
  delete VolumeMap;
  delete VolumeMap2;
}
